@@ -6,23 +6,23 @@ no warnings "experimental";
 use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT =
-  qw(ast_to_parser grammar_to_ast get_parser match spp_to_spp parse get_matcher match_matcher lint spp parse);
+  qw(ast_to_parser grammar_to_ast get_parser match
+     spp_to_spp parse lint spp parse lint_spp_ast);
 
-our $VERSION = '1.16';
+our $VERSION = '1.20';
 use Spp::Builtin;
-use Spp::IsChar;
+use Spp::Core;
 use Spp::Ast;
 use Spp::Grammar;
 use Spp::Cursor;
-use Spp::IsAtom;
-use Spp::LintAst qw(lint_ast ast_to_table);
-use Spp::Match qw(match_rule);
+use Spp::LintAst qw(lint_spp_ast);
+use Spp::Match  qw(match_rule);
 use Spp::OptAst qw(opt_spp_ast);
-use Spp::ToSpp qw(ast_to_spp);
+use Spp::ToSpp  qw(ast_to_spp);
 
 sub get_spp_parser {
    my $ast = get_spp_ast();
-   lint_ast($ast);
+   lint_spp_ast($ast);
    return ast_to_parser($ast);
 }
 
@@ -36,15 +36,15 @@ sub ast_to_parser {
 sub grammar_to_ast {
    my $text   = shift;
    my $parser = get_spp_parser();
-   my $match  = match($parser, $text, 0);
-   if (is_false($match)) { die $match->[1] }
+   my $match  = match($parser, $text);
+   if (is_false($match)) { error($match->[1]) }
    return opt_spp_ast($match);
 }
 
 sub get_parser {
    my $text = shift;
    my $ast  = grammar_to_ast($text);
-   lint_ast($ast);
+   lint_spp_ast($ast);
    return ast_to_parser($ast);
 }
 
@@ -54,7 +54,7 @@ sub eval_spp_ast {
    my $table  = $parser->[1];
    if (exists $table->{'text'}) {
       my $text = get_text($table->{'text'});
-      return match($parser, $text, 0);
+      return match($parser, $text);
    }
    return $ast;
 }
@@ -62,7 +62,7 @@ sub eval_spp_ast {
 sub get_text {
    my $rule = shift;
    if (is_atom_str($rule)) { return $rule->[1] }
-   die "could not get Rule text!";
+   error("could not get Rule text!");
 }
 
 sub repl {
@@ -72,30 +72,28 @@ sub repl {
       print ">> ";
       my $line = <STDIN>;
       exit() if $line eq "\n";
-      $line = trim($line);
-      my $match = match($parser, $line, 0);
+      my $match = match($parser, $line);
       if (is_false($match)) {
          print $match->[1];
       }
       else {
          my $ast = opt_spp_ast($match);
-         if (lint_ast($ast)) {
+         if (lint_spp_ast($ast)) {
             $ast = eval_spp_ast($ast);
-            say '.. ', to_json($ast);
+            say '.. ', to_json(clean_ast($ast));
          }
       }
    }
 }
 
 sub match {
-   my ($parser, $text, $mode) = @_;
+   my ($parser, $text) = @_;
    my ($door, $table) = @{$parser};
-   my $cursor = cursor($text, $table);
-   my $door_rule = $cursor->{'ns'}{$door};
-   $cursor->{'mode'} = $mode;
+   my $door_rule = $table->{$door};
+   my $cursor = Spp::Cursor->new($text, $table);
    my $match = match_rule($door_rule, $cursor);
    if (is_false($match)) {
-      my $max_report = max_report($cursor);
+      my $max_report = $cursor->max_report;
       return ['false', $max_report];
    }
    if (is_true($match)) { return $match }
@@ -108,7 +106,7 @@ sub match {
 sub update {
    my $grammar = get_spp_grammar();
    my $ast     = grammar_to_ast($grammar);
-   if (lint_ast($ast)) {
+   if (lint_spp_ast($ast)) {
       my $code = ast_to_perl_module($ast);
       if (-e 'Spp/Ast.pm') {
          rename('Spp/Ast.pm', 'Spp/Ast.pm.bak');
@@ -121,28 +119,28 @@ sub lint {
    my $grammar_file = shift;
    my $grammar_text = read_file($grammar_file);
    my $ast          = grammar_to_ast($grammar_text);
-   lint_ast($ast);
+   lint_spp_ast($ast);
 }
 
 sub spp_to_spp {
    my $str    = shift;
    my $parser = get_spp_parser();
-   my $match  = match($parser, $str, 0);
+   my $match  = match($parser, $str);
    if (is_false($match)) {
       say "Could not match!";
-      die $match->[1];
+      error($match->[1]);
    }
    my $ast = opt_spp_ast($match);
    return ast_to_spp($ast);
 }
 
 sub parse {
-   my ($grammar, $code, $mode) = @_;
+   my ($grammar, $code) = @_;
    my $parser = get_parser($grammar);
-   lint_parser($parser);
-   my $match = match($parser, $code, $mode);
-   if (is_false($match)) { die $match->[1] }
-   return to_json($match);
+   my $match = match($parser, $code);
+   if (is_false($match)) { error($match->[1]) }
+   return $match if is_true($match);
+   return to_json(clean_ast($match));
 }
 
 sub spp {
@@ -152,32 +150,14 @@ sub spp {
    return parse($grammar, $text);
 }
 
-sub get_matcher {
-   my $grammar = shift;
-   my $ast     = grammar_to_ast($grammar);
-   my $table   = ast_to_table($ast);
-   my $parser  = get_spp_parser();
-   return [$parser, $table];
-}
-
-sub match_matcher {
-   my ($matcher, $rule_text, $str) = @_;
-   my ($spp_parser, $table) = @{$matcher};
-   my $rule_spec    = "door = " . $rule_text;
-   my $rule_ast     = match($spp_parser, $rule_spec, 0);
-   if (is_false($rule_ast)) { die $rule_ast->[1] }
-   my $opt_rule_ast = opt_spp_ast($rule_ast);
-   my $rule         = $opt_rule_ast->[0][1];
-   my $cursor       = cursor($str, $table);
-   my $match        = match_rule($rule, $cursor);
-   return is_match($match);
-}
-
 sub ast_to_perl_module {
    my $ast = shift;
    my $str = <<'EOFF';
 package Spp::Ast;
 
+use 5.012;
+no warnings "experimental";
+
 use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(get_spp_ast);
