@@ -1,5 +1,8 @@
 package Spp::Grammar;
 
+use 5.012;
+no warnings "experimental";
+
 use Exporter;
 our @ISA    = qw(Exporter);
 our @EXPORT = qw(get_spp_grammar);
